
-- =================================================================================
--                          TauntMaster2                                          ==
--          http://www.curse.com/addons/wow/taunt-master-2                        ==
--                                                                                ==
-- =================================================================================
-- Author      : Tartarusspawn

local ResolveName = (GetSpellInfo(158300))
local TM2PlayerName = UnitName("player") or "Unknown"
	local currentSpec, currentSpecName
-- This function can return a substring of a UTF-8 string, properly handling
-- UTF-8 codepoints.  Rather than taking a start index and optionally an end
-- index, it takes the string, the start index and the number of characters
-- to select from the string.
--
-- UTF-8 Reference:
-- 0xxxxxx - ASCII character
-- 110yyyxx - 2 byte UTF codepoint
-- 1110yyyy - 3 byte UTF codepoint
-- 11110zzz - 4 byte UTF codepoint

local function utf8sub(str, start, numChars)
	local currentIndex = start
	while numChars > 0 and currentIndex <= #str do
		local char = string.byte(str, currentIndex)
		if char >= 240 then
			currentIndex = currentIndex + 4
		elseif char >= 225 then
			currentIndex = currentIndex + 3
		elseif char >= 192 then
			currentIndex = currentIndex + 2
		else
			currentIndex = currentIndex + 1
		end
		numChars = numChars - 1
	end
	return str:sub(start, currentIndex - 1)
end

local unitGUIeventHandlers = {
	-- Handle any events that don't accept a unit argument
	["PLAYER_ENTERING_WORLD"] = function  (self, unit)
		TauntMasterGUI_resize_frames(self)
		TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
	end,
	["PARTY_MEMBERS_CHANGED"] = function  (self, unit)
		TauntMasterGUI_resize_frames(self)
		TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
	end,
	["RAID_ROSTER_UPDATE"] = function  (self, unit)
		TauntMasterGUI_resize_frames(self)
		TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
	end,
	["GROUP_ROSTER_UPDATE"] = function  (self, unit)
		TauntMasterGUI_resize_frames(self)
		TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
	end,
	["PLAYER_TARGET_CHANGED"] = function  (self, unit)
		if UnitIsUnit(unit, "target") then
			self.unit.selected:Show()
		else
			self.unit.selected:Hide()
		end
	end,
	-- Handle any events that accept a unit argument
	["UNIT_MAXHEALTH"] = function  (self, unit)
		if self.unit.dead ~= UnitIsDeadOrGhost(unit) then
			TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
		else
			self.unit.healthBar:SetMinMaxValues(0, UnitHealthMax(unit))
			self.unit.healthBar:SetValue(UnitHealth(unit))
		end
	end,
	["UNIT_HEALTH"] = function  (self, unit)
		if self.unit.dead ~= UnitIsDeadOrGhost(unit) then
			TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
		else
			self.unit.healthBar:SetValue(UnitHealth(unit))
		end
	end,
	["UNIT_POWER"] = function  (self, unit)
		if self.unit.dead ~= UnitIsDeadOrGhost(unit) then
			TauntMasterGUI_UnitFrames_ResetPowerBar(self.unit, unit)
		else
			self.unit.powerBar:SetValue(UnitPower(unit))
		end
	end,
	["UNIT_MAXPOWER"] = function  (self, unit)
		if self.unit.dead ~= UnitIsDeadOrGhost(unit) then
			TauntMasterGUI_UnitFrames_ResetPowerBar(self.unit, unit)
		else
			self.unit.powerBar:SetMinMaxValues(0, UnitPowerMax(unit))
			self.unit.powerBar:SetValue(UnitPower(unit))
		end
	end,
	["UNIT_NAME_UPDATE"] = function  (self, unit)
		TauntMasterGUI_UnitFrames_ResetPowerBar(self.unit, unit)
	end,
	["UNIT_PORTRAIT_UPDATE"] = function  (self, unit)
		TauntMasterGUI_UnitFrames_ResetUnitButton(self.unit, unit)
	end,
	["UNIT_THREAT_SITUATION_UPDATE"] = function  (self, unit)
		local status = UnitThreatSituation(unit)
		local r, g, b = GetThreatStatusColor(status)
		self.unit.healthBar:SetStatusBarColor(r, g, b)
	end
}

function TauntMasterGUI_ShowFrame()
	--<Attribute name="point"               type="string"   value="TOP"/>
	--<Attribute name="columnAnchorPoint"   type="string"   value="LEFT"/>
	--TauntMasterUnitFrames_Header:SetAttribute( "point", "LEFT" )
	--TauntMasterUnitFrames_Header:SetAttribute( "columnAnchorPoint",  "TOP")
	--TauntMasterUnitFrames_Header:SetAllPoints(true)
	TauntMasterUnitFrames_Header:SetScale(1)
	TauntMasterUnitFrames_Header:Show()
	if resolveBar:IsShown() then
		--Do nothing
	else
		resolveBar_SetSpec()
	end
end
		--(frame:GetHeight()/32*100)*.25
function TauntMasterGUI_resize_frames(frame)
	--[[
	frame:SetWidth( TauntMaster2.db.char.gui.width-2 )
	frame:SetHeight( TauntMaster2.db.char.gui.height-2 )
	frame.unit.healthBar:SetSize( (frame:GetWidth()/100)*98, (frame:GetHeight()/100)*79 )
	frame.unit.powerBar:SetSize((frame:GetWidth()/100)*98, (frame:GetHeight()/100)*19 )
	frame.unit.oomicon:SetSize((frame:GetWidth()/100)*98, (frame:GetHeight()/100)*79 )
	TauntMasterGUI_resize_resolveframe(frame)
	--frame.Spell1:SetHeight( (frame:GetHeight()/32*100)*.25 )
	--frame.Spell2:SetHeight( (frame:GetHeight()/32*100)*.25 )
	--frame.Spell1:SetWidth((TauntMaster2.db.char.gui.width/63*100)*.25)
	--frame.Spell2:SetWidth((TauntMaster2.db.char.gui.width/63*100)*.25
	]]
end

function TauntMasterGUI_UnitFrames_ResetName(button, unit)
	local name = UnitName(unit) or "Unknown"
	local substring
	for length=#name, 1, -1 do
		substring = utf8sub(name, 1, length)
		button.name:SetText(substring)
		if button.name:GetStringWidth() <= (button.name:GetWidth() - 6 ) then
			return
		end
	end
	local status = UnitThreatSituation(unit)
	if status and status > 0 then
		local r, g, b = GetThreatStatusColor(status)
		button.healthBar:SetStatusBarColor(r, g, b)
	else
		button.healthBar:SetStatusBarColor(1, 1, 1)
	end
end

function TauntMasterGUI_UnitFrames_ResetHealthBar(button, unit)
	local class = select(2, UnitClass(unit)) or "WARRIOR"
	local classColor = RAID_CLASS_COLORS[class]
	local alive = not UnitIsDeadOrGhost(unit)
	if alive then
		button.name:SetTextColor(classColor.r, classColor.g, classColor.b)
		button.healthBar:SetMinMaxValues(0, UnitHealthMax(unit))
		button.healthBar:SetValue(UnitHealth(unit))
		button.dead = false
	else
		button.healthBar:SetStatusBarColor(0.3, 0.3, 0.3)
		button.healthBar:SetMinMaxValues(0, 1)
		button.healthBar:SetValue(1)
		button.dead = true
	end
end

function TauntMasterGUI_UnitFrames_ResetPowerBar(button, unit)
	local power = UnitPower(unit)
	local powermax = UnitPowerMax(unit)
	local powerType, powerToken = UnitPowerType(unit)
	local powerColor = PowerBarColor[powerToken]
	local alive = not UnitIsDeadOrGhost(unit)
	if alive then
		button.powerBar:SetStatusBarColor(powerColor.r, powerColor.g, powerColor.b)
	else
		button.powerBar:SetStatusBarColor(0.3, 0.3, 0.3)
		--button.unit.oomicon:Hide()
	end
	button.powerBar:SetMinMaxValues(0, powermax)
	button.powerBar:SetValue(power)
	button.oomicon:Hide()
end

function TauntMasterGUI_UnitFrames_ResetUnitButton(button, unit)
	TauntMasterGUI_UnitFrames_ResetHealthBar(button, unit)
	TauntMasterGUI_UnitFrames_ResetPowerBar(button, unit)
	TauntMasterGUI_UnitFrames_ResetName(button, unit)
	if UnitIsUnit(unit, "target") then
		button.selected:Show()
	else
		button.selected:Hide()
	end
	local status = UnitThreatSituation(unit)
	if status and status > 0 then
		local r, g, b = GetThreatStatusColor(status)
		button.healthBar:SetStatusBarColor(r, g, b)
	else
		button.healthBar:SetStatusBarColor(1, 1, 1)
	end
end


function TauntMasterGUI_UnitFrames_Frame_OnLoad(frame)
	--TauntMasterGUI_resize_frames(frame)
	-- Nudge the status bar frame levels down
	--self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	--self:RegisterEvent('ADDON_LOADED')
	--self:RegisterEvent('PLAYER_LOGIN')
	--self:RegisterEvent("UNIT_SPELLCAST_SENT")
	--self:RegisterEvent("PLAYER_REGEN_DISABLED")
	--self:RegisterEvent("PLAYER_REGEN_ENABLED")
	--self:RegisterEvent("UNIT_TARGET")
	--frame.oomicon = frame:GetName().."_TM_OOM_Icon"

	for event in pairs(unitGUIeventHandlers) do
		frame:RegisterEvent(event)
	end
	frame.unit.healthBar:SetFrameLevel(frame.unit:GetFrameLevel())
	frame.unit.powerBar:SetFrameLevel(frame.unit:GetFrameLevel())
	frame.unit:RegisterForDrag("LeftButton")
	frame.unit:RegisterForClicks("AnyUp")

	TauntMasterGUI_ResetResolveBar(frame)
	--TauntMasterGUI_resize_frames(frame)
	TauntMasterGUI_UnitFrames_Frame_SetAttributes(frame)
end

function TauntMasterGUI_UnitFrames_Frame_SetAttributes(frame)
	frame.unit:SetAttribute("useparent-unit", "true")


	frame.Spell1:SetTexture(select(3, GetSpellInfo(TauntMaster2.db.char.Spells[TauntMaster2.db.char.id]["LeftButton"])))
	frame.Spell2:SetTexture(select(3, GetSpellInfo(TauntMaster2.db.char.Spells[TauntMaster2.db.char.id]["RightButton"])))
	
	

	local SpellName1 = select(1, GetSpellInfo(TauntMaster2.db.char.Spells[TauntMaster2.db.char.id]["LeftButton"]))
	
	if (IsHelpfulSpell(SpellName1) )then 
		
		frame.unit:SetAttribute("unitsuffix1", "")
	else
		
		frame.unit:SetAttribute("unitsuffix1", "target")
	end
	frame.unit:SetAttribute("type1", "spell")
	frame.unit:SetAttribute( "spell1", GetSpellInfo(SpellName1) )
	
	local SpellName2 = select(1, GetSpellInfo(TauntMaster2.db.char.Spells[TauntMaster2.db.char.id]["RightButton"]))
	
	if (IsHelpfulSpell(SpellName2) )then 

		frame.unit:SetAttribute("unitsuffix1", "")
	else
		
		frame.unit:SetAttribute("unitsuffix2", "target")
	end
	frame.unit:SetAttribute("type2", "spell")
	frame.unit:SetAttribute("spell2", GetSpellInfo(SpellName2))
end

function TauntMasterGUI_UnitFrames_Frame_OnShow(button)
	local unit = button:GetAttribute("unit")
	if unit then
		local guid = UnitGUID(unit)
		if guid ~= button.guid then
			TauntMasterGUI_UnitFrames_ResetUnitButton(button.unit, unit)
			button.guid = guid
		end
	end

end

function TauntMasterGUI_UnitFrames_Frame_OnEvent(self, event, arg1, ...)
	local unit = self:GetAttribute("unit")
	if not unit then
		return
	end
	local handler = unitGUIeventHandlers[event]
	if handler then
		handler(self, unit)
	end
end


function TauntMasterGUI_UnitFrames_Button_OnDragStart(self, button)
	TauntMasterUnitFrames_Header:StartMoving()
	TauntMasterUnitFrames_Header.isMoving = true
end

function TauntMasterGUI_UnitFrames_Button_OnDragStop(self, button)
	if TauntMasterUnitFrames_Header.isMoving then
		TauntMasterUnitFrames_Header:StopMovingOrSizing()
	end
end

function TauntMasterGUI_oomicon(self, unit)
	local unitsClass = select(2, UnitClass(unit))
	if unitsClass == "PALADIN" or unitsClass == "PRIEST" or unitsClass == "SHAMAN" or unitsClass == "DRUID" or unitsClass == "WARLOCK"  or unitsClass == "MAGE" then
		local power = UnitPower(unit)
		local maxpower = UnitPowerMax(unit)
		local powerType = UnitPowerType(unit)
		if powerType == 0 then
			if power <= (maxpower * (30 / 100)) then
				self.unit.oomicon:Show()
			else
				self.unit.oomicon:Hide()
			end
		else
			self.unit.oomicon:Hide()
		end
	else
		self.unit.oomicon:Hide()
	end
end

function TauntMasterGUI_UnitFrames_OnUpdate(button, elapsed)
	local unit = button:GetAttribute("unit")
	TauntMasterGUI_oomicon(button, unit)
end

function TauntMasterGUI_ResetResolveBar()
end

function TauntMasterGUI_resize_resolveframe(frame)
	local unitCount = GetNumGroupMembers();
	local unitsPerColumn =  TauntMasterUnitFrames_Header:GetAttribute("unitsPerColumn");
	local numColumns;
	if ( unitCount > unitsPerColumn ) then
		numColumns = min( ceil(unitCount / unitsPerColumn), (self:GetAttribute("maxColumns") or 1) );
	else
		unitsPerColumn = numDisplayed;
		numColumns = 1;
	end
	resolveBar:SetSize( (frame:GetWidth() ) - 4, (frame:GetHeight()/100)*.5)
end

function resolveBar_Initialize(self)

	self:SetWidth(150);
	self:SetHeight(10);
	-- border
	self.border:ClearAllPoints();
	self.border:SetTexture("Interface\\CastingBar\\UI-CastingBar-Border-Small");
	self.border:SetWidth(0);
	self.border:SetHeight(49);
	self.border:SetPoint("TOPLEFT", -23, 20);
	self.border:SetPoint("TOPRIGHT", 23, 20);
	-- bordershield
	self.borderShield:ClearAllPoints();
	self.borderShield:SetWidth(0);
	self.borderShield:SetHeight(49);
	self.borderShield:SetPoint("TOPLEFT", -28, 20);
	self.borderShield:SetPoint("TOPRIGHT", 18, 20);
	-- text
	self.text:ClearAllPoints();
	self.text:SetWidth(0);
	self.text:SetHeight(16);
	self.text:SetPoint("TOPLEFT", 0, 4);
	self.text:SetPoint("TOPRIGHT", 0, 4);
	self.text:SetFontObject("SystemFont_Shadow_Small");
	-- icon
	self.icon:Show();
	-- bar spark
	self.barSpark.offsetY = 0;
	self.barSpark:Hide()
	-- bar flash
	self.barFlash:ClearAllPoints();
	self.barFlash:SetTexture("Interface\\CastingBar\\UI-CastingBar-Flash-Small");
	self.barFlash:SetWidth(0);
	self.barFlash:SetHeight(49);
	self.barFlash:SetPoint("TOPLEFT", -23, 20);
	self.barFlash:SetPoint("TOPRIGHT", 23, 20);
	self.barFlash:Hide()
	self:Hide()
	resolveBar_SetSpec()
	self:SetScale(1)
end
	
function resolveBar_SetSpec()
	currentSpec = currentSpec or GetSpecialization()
	currentSpecName = currentSpecName or (currentSpec and select(1, GetSpecializationInfo(currentSpec)) or "None")
	
	if (currentSpecName == 250) or (currentSpecName == 104) or (currentSpecName == 268) or (currentSpecName == 66) or (currentSpecName == 73)  then
		-- 250 - Blood(Death Knight),  104 - Guardian(Druid) , 268 - Brewmaster(Monk ), 66 - Protection(Paladin) ,  73 - Protection(Warrior)
		resolveBar:SetMinMaxValues(0, 240)
		resolveBar:SetValue(0)
		resolveBar:RegisterEvent("UNIT_AURA") 
		resolveBar:SetScript("OnEvent", function(resolveBar, event, ...)
			if (UnitAura("player", ResolveName, nil, "HELPFUL")) then
				local n,_,_,_,_,dur,expi,_,_,_,id,_,_,_,val1,val2,val3 = UnitAura("player", ResolveName, nil, "HELPFUL");
				local ResolveValue,DamageTaken
				ResolveValue = val1 or 0
				--ResolveValue = 240
				DamageTaken = val3 or 0
				if (DamageTaken > 0) then
					resolveBar:Show()
					resolveBar.text:SetText(string.format("%s%% (%s)",ResolveValue,DamageTaken))
					--resolveBar.text:SetText(string.format("%s%%",ResolveValue))
					resolveBar:SetValue(ResolveValue)
				else
					if UnitAffectingCombat("player") then
						-- Do Nothing
					else
						resolveBar:Hide()
					end
					resolveBar.text:SetText(string.format("%s%%",ResolveValue))
					resolveBar:SetValue(ResolveValue)
				end
			end
		end
			)
		--local TM2PlayerSpec = GetSpecializationInfo(GetSpecialization())
		-- 62 - Arcane(Mage)
	elseif (currentSpecName == 62) then
		resolveBar:SetStatusBarColor(0.0, 0.0, 0.75)
		resolveBar:SetMinMaxValues(0, 4)
		resolveBar.text:SetText(string.format("%s",0))
		resolveBar:SetValue(0)
		resolveBar:RegisterEvent("UNIT_AURA") 
		resolveBar:SetScript("OnEvent", function(resolveBar, event, ...)
			if (UnitDebuff("player", "Arcane Charge", nil, "HELPFUL")) then
				local n,_,_,_,_,dur,expi,_,_,_,id,_,_,_,val1,val2,val3 = UnitDebuff("player", "Arcane Charge");
				local ArcaneValue,DamageTaken
				ArcaneValue = val3 or 0
				if ArcaneValue then
					resolveBar:Show()
					--resolveBar.text:SetText(string.format("%s%% (%s)",ResolveValue,DamageTaken))
					resolveBar.text:SetText(string.format("%s",ArcaneValue))
					resolveBar:SetValue(ArcaneValue)
				else
					resolveBar:Hide()
					resolveBar.text:SetText(string.format("%s",ArcaneValue))
					resolveBar:SetValue(ArcaneValue)
				end
			else
				resolveBar:Hide()
				resolveBar.text:SetText(string.format("%s",0))
				resolveBar:SetValue(0)
			end
		end
			)
	else
		local power = UnitPower("player")
		local powermax = UnitPowerMax("player")
		local powerType, powerToken = UnitPowerType("player")
		local powerColor = PowerBarColor[powerToken]
		--resolveBar:SetStatusBarColor(powerColor.r, powerColor.g, powerColor.b)
		resolveBar:SetMinMaxValues(0, powermax)
		resolveBar:SetValue(power)
		
		resolveBar:RegisterEvent("UNIT_POWER") 
		resolveBar:SetScript("OnEvent", function(resolveBar, event, ...)

			local power = UnitPower("player")
			local powermax = UnitPowerMax("player")
			local powerType, powerToken = UnitPowerType("player")
			local powerColor = PowerBarColor[powerToken]
			local alive = not UnitIsDeadOrGhost("player")
			if alive and UnitAffectingCombat("player") then
				resolveBar:SetStatusBarColor(powerColor.r, powerColor.g, powerColor.b)
				resolveBar.text:SetText(string.format("%s",power))
				resolveBar:Show()
			else
				resolveBar:SetStatusBarColor(0.3, 0.3, 0.3)
				resolveBar.text:SetText(string.format("%s",power))
				resolveBar:Hide()
			end
			resolveBar:SetMinMaxValues(0, powermax)
			resolveBar:SetValue(power)
		end			)
	end
end





